/*
** isc_stdout.c                 Some nice-to-have functions for output..
**
** Copyright (c) 1991, 1992, 1993 Peter Eriksson and Per Cederqvist of the
**                    Lysator Academic Computer Association.
**
** history:
** 910305 pen      moved into separate file
*/

#include <stdio.h>
#ifdef HAVE_STRING_H
#  include <string.h>
#endif
#ifdef HAVE_STDARG_H
#  include <stdarg.h>
#endif
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>

#include "isc.h"


extern int UCB_printf( int (*fctn)(int chr), const char *format, ...);
/*extern void *memcpy(void *, const void *, int);*/

int
isc_putc(int           chr,
	 IscSession  * scb)
{
  if (scb->state != ISC_STATE_CONNECTING &&
      scb->state != ISC_STATE_RUNNING)
    return EOF;
  
  while (scb->sendindex == sizeof(scb->sendbuf))
    isc_flush(scb);

  return scb->sendbuf[scb->sendindex++] = chr;
}



int
isc_write(IscSession  * scb,
	  const void  * buffer,
	  size_t        length)
{
  const char  * bp;
  int           len;
  int           blen;
  int           clen;


  if (scb->state != ISC_STATE_CONNECTING &&
      scb->state != ISC_STATE_RUNNING)
    return EOF;
  
  bp   = (const char *) buffer;
  clen = length;
  
  while (clen > 0)
  {
    blen = sizeof(scb->sendbuf) - scb->sendindex;

    /* Make room in sendbuf */
    while (blen == 0)
    {
      isc_flush(scb);
      blen = sizeof(scb->sendbuf) - scb->sendindex;
    }

    len  = (clen > blen ? blen : clen);
    
    memcpy(scb->sendbuf+scb->sendindex, bp, len);
    scb->sendindex += len;
    clen -= len;
    bp   += len;
  }

  return length;
}
