// Cache of conf-statuses.
//
// $Id: conf-cache.cc,v 1.18 1995/11/28 22:37:13 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <stdlib.h>
#include "conf-cache.h"
#include "question.h"
#include "misc.h"
#include "error-codes.h"
#include "text-stat.h"
#include "recipient.h"
#include "async_text_created.h"

static Conf_stat bad_conf_stat;
static Conf_stat zero_conf_stat(KOM_CONF_ZERO);

Conf_cache::Conf_cache(Connection *c, Async_dispatcher *disp)
    : conn(c),
      async_disp(disp),
      cr_tag(disp->register_text_created_handler
	     (async_callback_method(this, &Conf_cache::new_text)))
{
}

Conf_cache::~Conf_cache()
{
    async_disp->unregister_text_created_handler(cr_tag);
}

Connection *
Conf_cache::connection() const
{
    return conn;
}

bool
Conf_cache::present(Conf_no cno) const
{
    return statusmap.present(cno);
}

//void
//Conf_cache::clear()
//{
//    cerr << "Unimp\n";
//    abort();
//}


Status
Conf_cache::prefetch(Conf_no cno)
{
    if (cno <= 0)
	return st_error;

    if (statusmap.present(cno))
    {
	if (statusmap[cno].errno() == KOM_NO_ERROR)
	    return st_ok;
	else
	    return st_error;
    }
    else
    {
	if (!pendingmap.present(cno))
	{
	    pendingmap[cno] = new get_conf_stat_question(conn, cno);
	    return st_pending;
	}
	else
	{
	    switch (pendingmap[cno]->status())
	    {
	    case st_pending:
		return st_pending;
	    case st_ok:
		statusmap[cno] = pendingmap[cno]->result();
		pendingmap.delete_entry(cno);
		return st_ok;
	    case st_error:
		statusmap[cno] = Conf_stat(pendingmap[cno]->error());
		pendingmap.delete_entry(cno);
		return st_error;
	    }
	}
    }
    abort();
}


Conf_stat
Conf_cache::get(Conf_no cno)
{
    if (cno <= 0)
	return zero_conf_stat;

    if (statusmap.present(cno))
    {
	Conf_stat FIXME = statusmap[cno];
	return FIXME;
    }
    else if (pendingmap.present(cno))
    {
	Conf_stat res;
	get_conf_stat_question *gtr = pendingmap[cno];
	if (gtr->receive() == st_ok)
	    res = gtr->result();
	else
	    res = Conf_stat(gtr->error());
	statusmap[cno] = res;
	pendingmap.delete_entry(cno);
	return res;
    }
    else		
    {
	// Not present in cache, and no outstanding question.
	Conf_stat res;
	get_conf_stat_question gt(conn, cno);
	if (gt.receive() == st_ok)
	    res = gt.result();
	else
	    res = Conf_stat(gt.error());
	statusmap[cno] = res;
	return res;
    }
    abort();
}

//void
//Conf_cache::remove(Conf_no cno)
//{
//    abort();
//}

void
Conf_cache::new_text(const Async_text_created &msg)
{
    Text_stat t = msg.text_stat();
    for (int ix = 0; ix < t.num_recipients(); ix++)
    {
	Recipient *recpt = t.recipient(ix);
	if (statusmap.present(recpt->recipient()))
	{
	    Conf_stat cs = statusmap[recpt->recipient()];
	    cs.inform_text_exists(recpt->local_no(), t.creation_time());
	}
    }
}

bool
Conf_cache::Ccmap::present(Conf_no c) const
{
    // contains() really should be a const member function. This is a 
    // workaround. The line below should be substituted.
    return ((Conf_cache::Ccmap*)this)->contains(c) ? true : false;
    // return contains(c) ? true : false;
}

Conf_stat &
Conf_cache::Ccmap::operator[](Conf_no c)
{
    return Conf_noConf_statCHMap::operator[](c);
}


Conf_cache::Ccmap::Ccmap()
    : Conf_noConf_statCHMap(bad_conf_stat, 50)
{
}

bool
Conf_cache::Ccreqmap::present(Conf_no c)
{
    return contains(c) ? true : false;
}

void
Conf_cache::Ccreqmap::delete_entry(Conf_no c)
{
    delete operator[](c);
    del(c);
}

get_conf_stat_question *&
Conf_cache::Ccreqmap::operator[](Conf_no c)
{
    return Conf_noget_conf_stat_question_ptrCHMap::operator[](c);
}

static get_conf_stat_question_ptr bad_ptr = NULL;

Conf_cache::Ccreqmap::Ccreqmap()
    : Conf_noget_conf_stat_question_ptrCHMap(bad_ptr)
{
}

Conf_cache::Ccreqmap::~Ccreqmap()
{
    for (Pix i = first(); i != 0; next(i))
    {
	delete contents(i);
    }
}
