// Create-on-demand map of mappings for conferences.
//
// $Id: map-cache.cc,v 1.8 1995/11/18 22:49:41 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "kom-types.h"
#include "misc.h"
#include "map-cache.h"
#include "Conf_no.Text_mapping.CHMap.h"
#include "async_dispatcher.h"
#include "async_text_created.h"
#include "text-stat.h"

Map_cache::Map_cache(const Conferences &c, Async_dispatcher *disp)
 : m_conn(c.connection()),
   m_conf_cache(c.read_conf_cache()),
   async_disp(disp),
   cr_tag(disp->register_text_created_handler
	  (async_callback_method(this, &Map_cache::new_text))),
   mapmap()
{
}

//Map_cache::Map_cache(const Map_cache&m)
//{
//    abort();
//}

static Text_mapping bad_text_mapping;

Map_cache::Mtmap::Mtmap()
    : Conf_noText_mappingCHMap(bad_text_mapping, 50)
{
}

//Map_cache& 
//Map_cache::operator=(const Map_cache&m)
//{
//    abort();
//}

Map_cache::~Map_cache()
{
    async_disp->unregister_text_created_handler(cr_tag);
}

#if 0
void
Map_cache::clear()
{
    // FIXME +++ Not yet implemented.
}
#endif

Connection*
Map_cache::connection() const
{
    return m_conn;
}

 
Text_mapping
Map_cache::get(Conf_no cno)
{
    Text_mapping &r = mapmap[cno];
    if (!r.valid())
	r = Text_mapping(Conference(cno, m_conf_cache));
    return r;
}

bool
Map_cache::Mtmap::present(Conf_no c) const
{
    return ((Map_cache::Mtmap*)this)->contains(c) ? true : false;
}

Text_mapping &
Map_cache::Mtmap::operator[](Conf_no c)
{
    return Conf_noText_mappingCHMap::operator[](c);
}

void
Map_cache::new_text(const Async_text_created &msg)
{
    Text_stat t = msg.text_stat();
    for (int ix = 0; ix < t.num_recipients(); ix++)
    {
	Recipient *recpt = t.recipient(ix);
	Text_mapping map = get(recpt->recipient());
	map.add(recpt->local_no(), t.text_no());
    }
}
