// Handle a membership in a conference.
//
// $Id: membership.cc,v 1.13 1995/11/18 22:49:47 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <stdlib.h>
#include <assert.h>

#include "prot-a-LyStr.h"
#include "membership.h"
#include "time-utils.h"


// ================================================================
//                 Membership_internal methods


Membership::Membership_internal::Membership_internal()
    : this_pers_no(0),
      kom_errno(0),
      m_conf_no(0),
      m_priority(0),
//      m_last_time_read(),
      m_last_text_read(0),
      read_texts()
{
}


Membership::Membership_internal::Membership_internal(Pers_no       pers_no,
						     prot_a_LyStr &str)
    : this_pers_no(pers_no),
      kom_errno(0)
{
    int   arrsize;
    int   i;

    m_last_time_read = parse_time(str);
    m_conf_no  = str.stol();
    m_priority = str.stol();
    m_last_text_read = str.stol();

    arrsize = str.parse_array_start();
    for (i = 0; i < arrsize; ++i) 
    {
	read_texts.append(str.stol());
    }
    str.parse_array_end(arrsize);
}


Membership::Membership_internal::Membership_internal(const 
						     Membership_internal &m)
    : refcount(m.refcount)
    , this_pers_no(m.this_pers_no)
    , kom_errno(m.kom_errno)
    , m_conf_no(m.m_conf_no)
    , m_priority(m.m_priority)
    , m_last_time_read(m.m_last_time_read)
    , m_last_text_read(m.m_last_text_read)
    , read_texts(m.read_texts)
{
}


const Membership::Membership_internal &
Membership::Membership_internal::operator =(const 
					  Membership::Membership_internal &m)
{
    refcount = m.refcount;
    this_pers_no = m.this_pers_no;
    kom_errno = m.kom_errno;
    m_conf_no = m.m_conf_no;
    m_priority = m.m_priority;
    m_last_time_read = m.m_last_time_read;
    m_last_text_read = m.m_last_text_read;
    read_texts = m.read_texts;

    return *this;
}

// ================================================================
//                     Membership methods


Membership::Membership()
{
    memb_int = NULL;
}


Membership::Membership(const Membership &m)
{
    if (m.memb_int != NULL)
    {
	m.memb_int->refcount++;
    }
    memb_int = m.memb_int;
}


Membership::Membership(Pers_no pno, prot_a_LyStr &prot_a_str)
{
    memb_int = new Membership_internal(pno, prot_a_str);
    memb_int->refcount = 1;
}


Membership::Membership(int errno)
{
    memb_int = new Membership_internal;
    memb_int->refcount = 1;
    memb_int->kom_errno = errno;
}


Membership::~Membership()
{
    if (memb_int != NULL)
    {    
	memb_int->refcount--;
	if (memb_int->refcount == 0)
	    delete memb_int;
    }
}


const Membership &
Membership::operator =(const Membership &m)
{
    if (&m != this)
    {
	if (memb_int != NULL)
	{
	    memb_int->refcount--;
	    if (memb_int->refcount == 0)
		delete memb_int;
	}

	memb_int = m.memb_int;
	if (memb_int != NULL)
	    memb_int->refcount++;
    }

    return *this;
}


// ================================================================


bool
Membership::valid() const
{
    return (bool) (memb_int != NULL && memb_int->kom_errno == 0);
}


int
Membership::errno() const
{
    assert(memb_int != NULL);
    return memb_int->kom_errno;
}


// ================================================================
//                            Access methods


bool
Membership::is_read(Local_text_no lto) const
{
    assert(valid());
    if (lto <= memb_int->m_last_text_read)
	return true;

    // FIXME:Casts due to non-const declaration of first() and next() in libg++
    for  (Pix i = ((Membership*)this)->memb_int->read_texts.first(); 
	  i != 0; ((Membership*)this)->memb_int->read_texts.next(i)) {
	if (((Membership*)this)->memb_int->read_texts(i) == lto)
	    return true;
    }

    return false;
}


Local_text_no
Membership::last_text_read() const
{
    assert(valid());
    return memb_int->m_last_text_read; 
}

long	   
Membership::num_read_after_last_text_read() const
{
    assert(valid());
    return ((Membership *)this)->memb_int->read_texts.length();
}
