// Fetch-on-demand cache of persons.
//
// $Id: pers-cache.cc,v 1.11 1995/11/18 22:49:50 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <stdlib.h>
#include "pers-cache.h"
#include "question.h"
#include "misc.h"
#include "error-codes.h"

static Pers_stat bad_pers_stat;
static Pers_stat zero_pers_stat(KOM_CONF_ZERO);

Pers_cache::Pers_cache(Connection *c)
{
    conn = c;
}

Pers_cache::~Pers_cache()
{
}

bool
Pers_cache::present(Pers_no cno) const
{
    return statusmap.present(cno);
}

//void
//Pers_cache::clear()
//{
//    cerr << "Unimp\n";
//    abort();
//}


Status
Pers_cache::prefetch(Pers_no pno)
{
    if (pno <= 0)
	return st_error;

    if (statusmap.present(pno))
    {
	if (statusmap[pno].errno() == KOM_NO_ERROR)
	    return st_ok;
	else
	    return st_error;
    }
    else
    {
	if (!pendingmap.present(pno))
	{
	    pendingmap[pno] = new get_person_stat_question(conn, pno);
	    return st_pending;
	}
	else
	{
	    switch (pendingmap[pno]->status())
	    {
	    case st_pending:
		return st_pending;
	    case st_ok:
		statusmap[pno] = pendingmap[pno]->result();
		pendingmap.delete_entry(pno);
		return st_ok;
	    case st_error:
		statusmap[pno] = Pers_stat(pendingmap[pno]->error());
		pendingmap.delete_entry(pno);
		return st_error;
	    }
	}
    }
    abort();
}


Pers_stat
Pers_cache::get(Pers_no pno)
{
    if (pno <= 0)
	return zero_pers_stat;

    if (statusmap.present(pno))
	return statusmap[pno];
    else if (pendingmap.present(pno))
    {
	Pers_stat res;
	get_person_stat_question *gtr = pendingmap[pno];
	if (gtr->receive() == st_ok)
	    res = gtr->result();
	else
	    res = Pers_stat(gtr->error());
	statusmap[pno] = res; 
	pendingmap.delete_entry(pno);
	return res;
    }
    else		
    {
	// Not present in cache, and no outstanding question.
	Pers_stat res;
	get_person_stat_question gt(conn, pno);
	if (gt.receive() == st_ok)
	    res = gt.result();
	else
	    res = Pers_stat(gt.error());
	statusmap[pno] = res;
	return res;
    }
    abort();
}

//void
//Pers_cache::remove(Pers_no pno)
//{
//    abort();
//}

bool
Pers_cache::Ppmap::present(Pers_no c) const
{
    // contains() really should be a const member function. This is a 
    // workaround. The line below should be substituted.
    return ((Pers_cache::Ppmap*)this)->contains(c) ? true : false;
    // return contains(c) ? true : false;
}

Pers_stat &
Pers_cache::Ppmap::operator[](Pers_no c)
{
    return Pers_noPers_statCHMap::operator[](c);
}


Pers_cache::Ppmap::Ppmap()
    : Pers_noPers_statCHMap(bad_pers_stat, 50)
{
}

bool
Pers_cache::Ppreqmap::present(Pers_no c)
{
    return contains(c) ? true : false;
}

void
Pers_cache::Ppreqmap::delete_entry(Pers_no c)
{
    delete operator[](c);
    del(c);
}

get_person_stat_question *&
Pers_cache::Ppreqmap::operator[](Pers_no c)
{
    return Pers_noget_person_stat_question_ptrCHMap::operator[](c);
}

static get_person_stat_question_ptr bad_ptr = NULL;

Pers_cache::Ppreqmap::Ppreqmap()
    : Pers_noget_person_stat_question_ptrCHMap(bad_ptr)
{
}

Pers_cache::Ppreqmap::~Ppreqmap()
{
    for (Pix i = first(); i != 0; next(i))
    {
	delete contents(i);
    }
}
