// Read texts in comment-order.
//
// $Id: read-order.cc,v 1.22 1995/12/30 16:25:31 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <assert.h>
#include <iostream.h>
#include <stdlib.h>

#include "conference.h"
#include "connection.h"
#include "get_membership_q.h"
#include "kom-types.h"
#include "maps.h"
#include "membership.h"
#include "misc.h"
#include "read-order.h"
#include "text-mapping.h"
#include "text.h"

// Helper class

Text_no
Comment_stack::pop()
{
    return remove_front();
}

void
Comment_stack::push(Text_no tno)
{
    prepend(tno);
}

void
Comment_stack::remove(Text_no tno)
{
    for (Pix ix = first(); ix != 0; next(ix)) {
	if ((*this)(ix) == tno) {
	    del(ix, -1);
	}
    }
}

Read_item
Extended_comment_stack::pop()
{
    return remove_front();
}

void
Extended_comment_stack::push(const Read_item &r)
{
    prepend(r);
}

void
Extended_comment_stack::remove(Text_no tno)
{
    for (Pix ix = first(); ix != 0; next(ix)) {
	if ((*this)(ix).text_no == tno) {
	    del(ix, -1);
	}
    }
}

Work_quota::Work_quota(int required_texts,
		       int required_texts_in_conf,
		       int required_confs,
		       int required_maps,
		       int required_maps_in_conf,
		       int required_maps_conf,
		       int max_questions)
    : text_quota(required_texts),
      texts_in_conf_quota(required_texts_in_conf),
      texts_in_this_conf_quota(required_texts_in_conf),
      confs_quota(required_confs),
      map_quota(required_maps),
      maps_in_conf_quota(required_maps_in_conf),
      maps_in_this_conf_quota(required_maps_in_conf),
      map_confs_quota(required_maps_conf),
      question_quota(max_questions)
{
    if (map_quota < text_quota)
	map_quota = text_quota;
    if (maps_in_conf_quota < texts_in_conf_quota) 
    {
	maps_in_conf_quota = texts_in_conf_quota;
	maps_in_this_conf_quota = texts_in_conf_quota;
    }
}

Work_quota::Work_quota(int required_texts,
		       int required_maps,
		       int max_questions)
    : text_quota(required_texts),
      texts_in_conf_quota(0),
      texts_in_this_conf_quota(0),
      confs_quota(0),
      map_quota(required_maps),
      maps_in_conf_quota(0),
      maps_in_this_conf_quota(0),
      map_confs_quota(0),
      question_quota(max_questions)
{
    if (map_quota < text_quota)
	map_quota = text_quota;
}

Work_quota::Work_quota()
    : text_quota(20),
      texts_in_conf_quota(4),
      texts_in_this_conf_quota(4),
      confs_quota(20),
      map_quota(100),
      maps_in_conf_quota(50),
      maps_in_this_conf_quota(50),
      map_confs_quota(40),
      question_quota(15)
{
    if (map_quota < text_quota)
	map_quota = text_quota;
    if (maps_in_conf_quota < texts_in_conf_quota) 
    {
	maps_in_conf_quota = texts_in_conf_quota;
	maps_in_this_conf_quota = texts_in_conf_quota;
    }
}

Work_quota::~Work_quota()
{
}

void 
Work_quota::bump_question(int q)
{
    question_quota -= q;
    if (question_quota < 0)
	question_quota = 0;
}

bool 
Work_quota::done() const
{
    return question_quota == 0 || (text_quota == 0 && confs_quota == 0
				   && map_quota == 0 && map_confs_quota == 0);
}

bool 
Work_quota::conf_done() const
{
    return (question_quota == 0
	    || (texts_in_this_conf_quota == 0 && text_quota == 0));
}

bool 
Work_quota::conf_mapped() const
{
    return (question_quota == 0
	    || (maps_in_this_conf_quota == 0 && map_quota == 0));
}

void 
Work_quota::bump_known_texts(int texts)
{
    text_quota -= texts;
    if (text_quota < 0)
	text_quota = 0;
    texts_in_this_conf_quota -= texts;
    if (texts_in_this_conf_quota < 0)
	texts_in_this_conf_quota = 0;
}

void 
Work_quota::bump_known_mappings(int texts)
{
    map_quota -= texts;
    if (map_quota < 0)
	map_quota = 0;
    maps_in_this_conf_quota -= texts;
    if (maps_in_this_conf_quota < 0)
	maps_in_this_conf_quota = 0;
}

void 
Work_quota::change_conf()
{
    if (confs_quota > 0)
    {
	if (texts_in_this_conf_quota != texts_in_conf_quota) 
	{
	    if (--confs_quota > 0)
		texts_in_this_conf_quota = texts_in_conf_quota;
	    else
		texts_in_this_conf_quota = 0;
	}
	else
	    texts_in_this_conf_quota = texts_in_conf_quota;
    }
    else
	texts_in_this_conf_quota = 0;

    if (map_confs_quota > 0)
    {
	if (maps_in_this_conf_quota != maps_in_conf_quota) 
	{
	    if (--map_confs_quota > 0)
		maps_in_this_conf_quota = maps_in_conf_quota;
	    else
		maps_in_this_conf_quota = 0;
	}
	else
	    maps_in_this_conf_quota = maps_in_conf_quota;
    }
    else
	maps_in_this_conf_quota = 0;

    if (map_confs_quota == 0 && confs_quota > 0)
	map_confs_quota = 1;
    if (maps_in_this_conf_quota < texts_in_this_conf_quota)
	maps_in_this_conf_quota = texts_in_this_conf_quota;
    if (map_quota < text_quota)
	map_quota = text_quota;
}

int
Work_quota::questions_left() const
{
    return question_quota;
}

// ================================================================ 
//                    Constructors and Destructors                  


Read_order::Read_order(Session *sess, Conf_no cno, Pers_no reader)
    : session(sess), persons(sess->persons()), 
      conferences(sess->conferences()),
      texts(sess->texts()),
      map(sess->maps()[cno]),
      conf_no(cno), pers_no(reader), pending_membership_question(NULL),
      membership(),
      first_unchecked(0),
      unread_texts()
{
}
 

Read_order::~Read_order()
{
    delete pending_membership_question;
}


// ================================================================


// Returns true if anything was received.
bool
Read_order::prefetch_map(Work_quota *work)
{
    bool got_anything = false;

    if (work->conf_mapped())
	return got_anything;
    Local_text_no lno = 0;
    Conference conf = conferences[conf_no];
    if (!membership.valid() && pending_membership_question == NULL)
    {
	pending_membership_question = 
	    new get_membership_question(conferences.connection(), 
					pers_no, conf_no);
    }

    switch (conf.prefetch())
    {
    case st_pending:
	// We can't do anything sensible unless we have the confstat.
	work->bump_question();
	if (pending_membership_question != NULL
	    && pending_membership_question->status() == st_pending)
	    work->bump_question();
	return got_anything;

    case st_error:
	// Now what? +++FIXME///
	// (Note: This case CAN occur if the conference is being
	// deleted during a session).
	return got_anything;

    case st_ok:
        {
	    Local_text_no first_local = conf.first_local_no();
	    Local_text_no limit = first_local + conf.no_of_texts();

	    work->bump_known_mappings(unread_texts.length());
	    if (!work->conf_mapped() && first_unchecked < limit)
	    {
		if (pending_membership_question != NULL)
		{
		    switch (pending_membership_question->status())
		    {
		    case st_pending:
			work->bump_question();
			return got_anything;
			
		    case st_error:
			// Now what? +++FIXME///
			return got_anything;
		    
		    case st_ok:
			receive_membership();
			break;
		    
		    default:
			abort();
		    }
		}

		if (first_unchecked < first_local)
		    first_unchecked = first_local;
		
		while (!removed.empty()
		       && ((lno = removed(removed.first())) < first_unchecked))
		    removed.del(lno);

		for (;
		     first_unchecked < limit && !work->conf_mapped();
		     first_unchecked++)
		{
		    if (!removed.empty()
			&& removed(removed.first()) == first_unchecked)
		    {
			removed.del(first_unchecked);
		    }
		    else if (!membership.is_read(first_unchecked))
		    {
			assert(!removed.contains(first_unchecked));
			// FIXME+++: The "50" on the next line used to be
			// "texts", but that caused a very large question
			// to be issued. The map question should be split
			// into severaly small chunks, but I'm not sure
			// where the splitting should be done.
		        // Temporarily, do it this way.
			
			if (map.prefetch(first_unchecked, 50) == st_pending)
			{
			    work->bump_question();
			    return got_anything;
			}

			Text_no tno = map[first_unchecked];
			if (tno != 0) 
			{
			    unread_texts.add(tno);
			    work->bump_known_mappings();
			    got_anything = true;
			}
		    }
		    else
		    {
			assert(!removed.contains(first_unchecked));
		    }
		}
	    }
	}
	break;

    default:
	abort();
    }

    return got_anything;
}

void
Read_order::prefetch_recurse(const Text_no &node,
			     Text_noAVLSet *stoplist,
			     Work_quota *work)
{
    // FIXME: This function keeps on recursing even if it reaches
    // texts that the user already has read.  That is a bug (unless
    // we want to prefetch the entire tree always, in case the user
    // wants to jump).

    if (stoplist->contains(node))
	return;
    if (work->conf_done())
	return;

    Text t = texts[node];
    int q = t.prefetch_for_display(work->questions_left(), session);

    if (q == 0)
    {
	assert(t.present());
	stoplist->add(node);
	work->bump_known_texts();
    }
    else 
    {
	work->bump_question(q);
    }

    if (t.present() && t.exists())
    {
	if (t.local_no_recipient(conf_no) == 0)	// Not present in this conf.
	    return;

	int ix;

	for (ix = 0; !work->conf_done() && ix < t.num_footnote_in(); ix++)
	    prefetch_recurse(t.footnote_in(ix), stoplist, work);
	
	for (ix = 0; !work->conf_done() && ix < t.num_comment_in(); ix++)
	    prefetch_recurse(t.comment_in(ix), stoplist, work);
    }
    return;
}

Status
Read_order::prefetch(Work_quota *work)
{
    // FIXME: This function keeps on recursing even if it reaches
    // texts that the user already has read.  That is a bug (unless
    // we want to prefetch the entire tree always, in case the user
    // wants to jump).
    Pix ix;
    Text_noAVLSet stoplist;
    int q = work->questions_left();

    for (ix = pending_footnotes.first(); ix != 0; pending_footnotes.next(ix))
	prefetch_recurse(pending_footnotes(ix), &stoplist, work);

    for (ix = pending_comments.first(); ix != 0; pending_comments.next(ix))
	prefetch_recurse(pending_comments(ix), &stoplist, work);

    for (ix = unread_texts.first();
	 ix != 0 && !work->conf_done();
	 unread_texts.next(ix))
    {
	prefetch_recurse(unread_texts(ix), &stoplist, work);
    }

    if (prefetch_map(work))
	return prefetch(work);

    if (q == work->questions_left())
	return st_ok;
    else
	return st_pending;
}
			
Prompt
Read_order::next_prompt()
{
    while (prefetch(&Work_quota(1, 0, 10)) == st_pending)
    {	
	switch (Connection::drain(100))	// FIXME: configurable timeout.
	{
	case st_pending:
	    return TIMEOUT_PROMPT;
	case st_error:
	    return PROMPT_ERROR;
	case st_ok:
	    break;
	}
    }

    if (!pending_footnotes.empty())
	return NEXT_FOOTNOTE;
    else if (!pending_comments.empty())
	return NEXT_COMMENT;
    else if (unread_texts.empty())
	return NO_MORE_TEXT;
    else
	return NEXT_TEXT;
}


//
// Return the next text to be read within this conference.
// Return 0 if there is no more unread text.
//

Text_no
Read_order::next_text(bool remove)
{
    Text_no tno = 0;
    
    if (need_one_mapped() == st_error)
	return 0;

    if (remove)
	if (!pending_footnotes.empty())
	    tno = pending_footnotes.pop();
	else if (!pending_comments.empty())
	    tno = pending_comments.pop();
	else if (!unread_texts.empty())
	{
	    tno = unread_texts(unread_texts.first());
	    unread_texts.del(tno);
	}
        else
	    tno = 0;
    else
	if (!pending_footnotes.empty())
	    tno = pending_footnotes.front();
	else if (!pending_comments.empty())
	    tno = pending_comments.front();
	else if (!unread_texts.empty())
	    tno = unread_texts(unread_texts.first());
        else
	    tno = 0;

    if (remove && tno != 0)
    {
	// Add comments/footnotes:
	Text t = texts[tno];

	if (t.exists())
	{
	    Text child;
	    int i;
	    Local_text_no lno;

	    // ...prefetch the statuses
	    for (i = t.num_footnote_in() -1; i >= 0; i--)
		texts[t.footnote_in(i)].prefetch();

	    for (i = t.num_comment_in() -1; i >= 0; i--)
		texts[t.comment_in(i)].prefetch();

	    // ...and receive them.
	    for (i = t.num_footnote_in() -1; i >= 0; i--)
	    {
		child = texts[t.footnote_in(i)];
		if (child.exists()
		    && (lno = child.local_no_recipient(conf_no)) != 0
		    && !removed.contains(lno)
		    && !membership.is_read(lno))
		{
		    Text_no footnote_number = t.footnote_in(i);
		    pending_footnotes.push(footnote_number);
		    unread_texts.add(footnote_number);
		}
	    }

	    for (i = t.num_comment_in() -1; i >= 0; i--)
	    {
		child = texts[t.comment_in(i)];
		if (child.exists()
		    && (lno = child.local_no_recipient(conf_no)) != 0
		    && !removed.contains(lno)
		    && !membership.is_read(lno))
		{
		    Text_no comment_number = t.comment_in(i);
		    pending_comments.push(comment_number);
		    unread_texts.add(comment_number);
		}
	    }
	    remove_text(tno, t.local_no_recipient(conf_no));
	}
	else
	    remove_text(tno, 0);
    }
    return tno;
}

void
Read_order::remove_text(Text_no tno,
			Local_text_no lno)
{
    if (lno == first_unchecked + 1)
    {
	Local_text_no del_no = 0;
	first_unchecked++;

	while (!removed.empty()
	       && (del_no = removed(removed.first())) < first_unchecked)
	    removed.del(del_no);

	while (!removed.empty()
	       && (del_no = removed(removed.first())) == first_unchecked)
	{
	    removed.del(del_no);
	    first_unchecked++;
	}
    }
    else if (lno > first_unchecked)
	removed.add(lno);

    pending_comments.remove(tno);
    pending_footnotes.remove(tno);
    // FIXME: Why this loop? Can we not check if it is present and remove
    // it in O(log n)? After all, unread_texts is sorted...
    for (Pix ix = unread_texts.first(); ix != 0; unread_texts.next(ix)) {
	if (unread_texts(ix) == tno) {
	    unread_texts.del(tno);
	    break;
	}
    }
}

bool
Read_order::empty()
{
    need_one_mapped();
    return unread_texts.empty();
}


long
Read_order::min_num_unread()
{
    return unread_texts.length();
}

 
void
Read_order::receive_membership()
{
    membership = pending_membership_question->result();
    delete pending_membership_question;
    pending_membership_question = NULL;
}    

long
Read_order::max_num_unread()
{
    Conference conf = conferences[conf_no];
    if (!conf.present())
	return LONG_MAX;

    if (pending_membership_question != NULL
	&& pending_membership_question->status() == st_ok)
    {
	receive_membership();
    }

    if (!membership.valid())
    {
	// FIXME: We could subtract any texts here that we know are
	// deleted. Such information may exist in the text-mapping.
	return conf.no_of_texts();
    }

    long result = unread_texts.length();

    Local_text_no limit = conf.first_local_no() + conf.no_of_texts();

#if 0
    // This is SLOW when you have several thousand unread in I]M.
    for (Local_text_no ln = ((first_unchecked > membership->last_text_read())
			     ? first_unchecked : membership->last_text_read());
	 ln < limit; 
	 ln++)
    {
	if (!membership->is_read(ln))
	{
	    // +++ Should check if we know that this text is deleted.
	    result++;
	}
    }
#else
    if (first_unchecked > membership.last_text_read())
	result += limit - first_unchecked;
    else
	result += (limit - membership.last_text_read()
		   - membership.num_read_after_last_text_read());
#endif

    return result;    
}

Conf_no
Read_order::confno() const
{
    return conf_no;
}

Read_order_iterator
Read_order::begin(int wanted_texts)
{
    return Read_order_iterator(this, wanted_texts);
}

Read_order_iterator
Read_order::end()
{
    return Read_order_iterator(this, 0);
}

Status
Read_order::need_one_mapped()
{
    while (prefetch(&Work_quota(1,0,10)) == st_pending)
	if (Connection::drain(100) == st_error)
	    return st_error;
    return st_ok;
}

// Read_order_iterator

Read_order_iterator::Read_order_iterator()
    : unread_texts(),
      pending_footnotes(),
      pending_comments(),
      removed(),
      texts_left(0),
      current(Read_item(0, 0)),
      ro(NULL)
{
    // Any attempt to use this iterator will result in a crash.
}

Read_order_iterator::~Read_order_iterator()
{
    unread_texts.clear();
    removed.clear();
}

Read_order_iterator::Read_order_iterator(const Read_order_iterator &iter)
    : unread_texts(),
      pending_footnotes(iter.pending_footnotes),
      pending_comments(iter.pending_comments),
      removed(),
      texts_left(iter.texts_left),
      current(iter.current),
      ro(iter.ro)
{
    unread_texts |= const_cast<Read_order_iterator*>(&iter)->unread_texts;
    removed |= const_cast<Read_order_iterator*>(&iter)->removed;
}

bool
Read_order_iterator::operator==(const Read_order_iterator &iter) const
{
    assert(ro != NULL);
    assert(iter.ro != NULL);
    assert(ro == iter.ro);
    return current.text_no == iter.current.text_no;
}

bool
Read_order_iterator::operator!=(const Read_order_iterator &iter) const
{
    assert(ro != NULL);
    assert(iter.ro != NULL);
    assert(ro == iter.ro);
    return current.text_no != iter.current.text_no;
}

Read_order_iterator &
Read_order_iterator::operator=(const Read_order_iterator &iter)
{
    unread_texts.clear();
    unread_texts |= const_cast<Read_order_iterator*>(&iter)->unread_texts;
    pending_footnotes = iter.pending_footnotes;
    pending_comments = iter.pending_comments;
    removed.clear();
    removed |= const_cast<Read_order_iterator*>(&iter)->removed;
    texts_left = iter.texts_left;
    current = iter.current;
    ro = iter.ro;

    return *this;
}

Read_item
Read_order_iterator::operator*()
{
    assert(texts_left > 0);
    assert(current.nesting_level >= 0);
    assert(current.text_no != 0);

    return current;
}

Read_order_iterator &
Read_order_iterator::operator++()
{
    assert(texts_left > 0);

    if (texts_left == 1)
    {
	assert (current.text_no != 0);
	// This is becoming a past-the-end iterator
	current.text_no = 0;
	return *this;
    }

    if (current.text_no == 0)
    {
	// This iterator is currently being initialized.  It
	// points before-the-start.  This condition is never
	// exposed to the user of the class; it exists while
	// the iterator is being constructed.
	assert(pending_footnotes.empty());
	assert(pending_comments.empty());
    }

    current.text_no = 0;

    if (!pending_footnotes.empty())
	current = pending_footnotes.pop();
    else if (!pending_comments.empty())
	current = pending_comments.pop();
    else if (!unread_texts.empty())
    {
	current.text_no = unread_texts(unread_texts.first());
	current.nesting_level = 0;
	unread_texts.del(current.text_no);
    }
    else
	abort();

    assert(current.text_no != 0);

    Text t = ro->texts[current.text_no];
    if (t.exists())
    {
	Text t = ro->texts[current.text_no];
	Text child;
	int i;
	Local_text_no lno;

	// ...prefetch the statuses
	for (i = t.num_footnote_in() -1; i >= 0; i--)
	    ro->texts[t.footnote_in(i)].prefetch();

	for (i = t.num_comment_in() -1; i >= 0; i--)
	    ro->texts[t.comment_in(i)].prefetch();

	// ...and receive them.
	for (i = t.num_footnote_in() -1; i >= 0; i--)
	{
	    child = ro->texts[t.footnote_in(i)];
	    if (child.exists()
		&& (lno = child.local_no_recipient(ro->conf_no)) != 0
		&& !removed.contains(lno)
		&& !ro->membership.is_read(lno))
	    {
		Text_no footnote_number = t.footnote_in(i);
		pending_footnotes.push(Read_item(footnote_number,
						 current.nesting_level+1));
		unread_texts.add(footnote_number);
	    }
	}

	for (i = t.num_comment_in() -1; i >= 0; i--)
	{
	    child = ro->texts[t.comment_in(i)];
	    if (child.exists()
		&& (lno = child.local_no_recipient(ro->conf_no)) != 0
		&& !removed.contains(lno)
		&& !ro->membership.is_read(lno))
	    {
		Text_no comment_number = t.comment_in(i);
		pending_comments.push(Read_item(comment_number,
						current.nesting_level+1));
		unread_texts.add(comment_number);
	    }
	}
    }

    pending_comments.remove(current.text_no);
    pending_footnotes.remove(current.text_no);
    // FIXME: Why this loop? Can we not check if it is present and remove
    // it in O(log n)? After all, unread_texts is sorted...
    for (Pix ix = unread_texts.first(); ix != 0; unread_texts.next(ix)) {
	if (unread_texts(ix) == current.text_no) {
	    unread_texts.del(current.text_no);
	    break;
	}
    }

    --texts_left;

    return *this;
}

Read_order_iterator
Read_order_iterator::operator++(int)
{
    Read_order_iterator tmp = *this;
    ++*this;
    return tmp;
}
    
Read_order_iterator::Read_order_iterator(Read_order *read_order,
					 int wanted_texts)
    : unread_texts(),
      pending_footnotes(),
      pending_comments(),
      removed(read_order->removed),
      texts_left(wanted_texts+1), // Compensate for the initial "hidden" step.
      current(Read_item(0, 0)),
      ro(read_order)
{
    if (wanted_texts == 0)	// The past-the-end value.
	texts_left = 0;

    while (ro->prefetch(&Work_quota(wanted_texts, 0,0, wanted_texts, 0,0, 20))
	   == st_pending)
    {
	if (Connection::drain(100) == st_error)
	{
	    texts_left = 0;
	    return;
	}
    }

    unread_texts |= ro->unread_texts;
    if (unread_texts.length() + 1 < texts_left)
	texts_left = unread_texts.length() + 1;

    if (texts_left > 0)
	++*this;			// Step forward to the first text.
}

// read-order.cc ends here
