// Highlevel handling of a session to a LysKOM server.
//
// $Id: session.h,v 1.28 1996/02/11 20:08:15 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef KOMXX_SESSION_H
#define KOMXX_SESSION_H

#include <stdlib.h>
#include <iostream.h>

#include "array.h"
#include "LyStr.h"
#include "kom-types.h"
#include "who-info.h"
#include "micro-conf.h"
#include "conf-z-info.h"
#include "connection.h"
#include "async_dispatcher.h"
#include "mark.h"

#include "obsolete.h"		// For who_is_on_question.

#include "conference.h"
#include "person.h"
#include "maps.h"
#include "text.h"
#include "membership.h"

class Membership_cache;

// +++FIXME: Hur skall felhanteringen g} till?  Vi vill kunna f} ut
//           en felkod om n}got g}r fel.

class Session {
  public:
    Session(Connection *);
    ~Session();

    friend ostream &operator <<(ostream &, const Session &);

    Conferences &  conferences();
    Persons     &  persons();
    Maps        &  maps();
    Texts       &  texts();
    Pers_no        my_login() const;
    Session_no     my_session() const;

    // Create a new person.  If RESULT is non-NULL, the new person number will
    // be stored there.  An automatic login is performed if you are not already
    // logged in.  If AUTOLOGIN is non-NULL, it will be set if an auto-login
    // was performed.
    Status              create_person(const LyStr &name, const LyStr &password,
				      Pers_no *result = NULL, 
				      bool *autologin = NULL);

    Status              login(Pers_no, const LyStr &, bool invisible = false);
    Status              logout();
    Status              add_member(Conf_no cno, Pers_no reader,
				   unsigned char priority, 
				   unsigned short placement);
    Status              send_message(Pers_no recipient, const LyStr &message);
    struct tm           get_time(); 
    Status              change_what_i_am_doing(const LyStr &str);
    Array<Who_info>     who_is_on();
    bool                is_member(Pers_no person, Conf_no conf);
    Membership          get_membership(Pers_no person, Conf_no conf);
    Array<Conf_z_info>  lookup_z_name(const LyStr &pattern,
				      bool want_persons =true,
				      bool want_confs =true);
    Array<Mark>         get_marks();

    // Obsolete:
    Array<Micro_conf>   lookup_name(const LyStr &pattern);
    Array<Pers_no>      regexp_lookup_persons(const LyStr &pattern);

    Async_dispatcher  & async_dispatcher();
    Status              handle_async(int this_many = 1);

  private:
    Connection  * conn;
    Pers_no       logged_in_person;
    Session_no    session_number; // 0 if not yet known

    Conferences * all_confs;
    Persons     * all_persons;
    Maps        * all_maps;
    Texts       * all_texts;
    Membership_cache *all_memberships;

    Async_dispatcher  * m_async_dispatcher;

    // Not implemented, since it doesn't make sense to copy sessions.
    Session(const Session &);
    const Session & operator = (const Session &);
};


#endif
