// Fetch-on-demand cache for texts.
//
// $Id: text-mass-cache.h,v 1.6 1995/11/18 22:50:22 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef KOMXX_TEXT_MASS_CACHE_H
#define KOMXX_TEXT_MASS_CACHE_H

#include <iostream.h>
#include <DLList.h>
#include <Pix.h>

#include "kom-types.h"
#include "misc.h"
#include "LyStr.h"


class Connection;

class Text_mass_cache {
  public:
    Text_mass_cache(Connection *c, int sz = 25);
    Text_mass_cache(const Text_mass_cache &tmc);
    ~Text_mass_cache();

    const Text_mass_cache & operator = (const Text_mass_cache &);

    friend ostream &operator <<(ostream &, const Text_mass_cache &);

    // Methods working on the entire cache:
    void        set_size(int sz);
    bool        present(Text_no tno);
    bool        empty();
    void        clear();
    Connection *connection() const;

    // Methods working on individual texts.
    Status      prefetch(Text_no tno);
    LyStr     * get(Text_no tno);
    void        remove(Text_no tno);

  private:
    struct Cache_item {
	Text_no   text_no;
	LyStr   * text;

	Cache_item();
	~Cache_item();
    };

    Connection        * conn;
    int                 size;
    DLList<Text_mass_cache::Cache_item>  lru_cache;

    // Find an entry in the cache.  Return a Pix.
    Pix find(Text_no tno);

    // Remove the least recently used entry in the cache.
    void lru_remove();

    // Make the entry pointed at by WHERE the most recently used.
    void lru_update(Pix where);

};
#endif
