// Text status objects.
//
// $Id: text-stat.h,v 1.14 1995/11/18 22:50:25 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef KOMXX_TEXT_STAT_H
#define KOMXX_TEXT_STAT_H

#include <time.h>
#include <DLList.h>

#include "array.h"
#include "kom-types.h"
#include "recipient.h"
#include "memcheck.h"

class Uplink;
class prot_a_LyStr;

/*  Struct for text status  */
class Text_stat MEMCHECK {
  public:
    Text_stat();
    Text_stat(const Text_stat&);
    Text_stat(Text_no tno, prot_a_LyStr &prot_a_string);
    Text_stat(int errno);
    Text_stat &operator =(const Text_stat &);
    ~Text_stat();

    int         errno()          const;

    Text_no     text_no()        const;
    struct tm   creation_time()  const;
    Pers_no     author()         const;
    long        num_lines()      const;
    long        num_chars()      const;
    int         num_marks()      const;
    int         num_recipients() const;
    Recipient*  recipient(int)   const;
    int		num_comment_in() const;
    Text_no	comment_in(int)	 const;
    int		num_footnote_in()const;
    Text_no	footnote_in(int) const;
    int		num_uplinks()	 const;
    Uplink*	uplink(int)	 const;

    void add_footnote_in(Text_no t);
    void add_comment_in(Text_no t);
  private:
    struct Text_stat_internal {
      public:
	Text_stat_internal();
	Text_stat_internal(Text_no text_no, prot_a_LyStr &prot_a_str);
	Text_stat_internal(const Text_stat_internal &);
	~Text_stat_internal();
	const Text_stat_internal &operator =(const Text_stat_internal &);

	int refcount;
	int kom_errno;

	Text_no           number;
	struct tm	  ctime;
	Pers_no		  auth;	
	unsigned short	  no_of_lines;
	unsigned long  	  no_of_chars;
	unsigned short	  no_of_marks;	
	Array<Recipient*> recipients; // ...and cc_recipients
	Array<Text_no>	  comments_in;
	Array<Text_no>	  footnotes_in;
	Array<Uplink*>	  uplinks; // footn_to, comm_to
    };
    bool valid() const;
    Text_stat_internal *tsi;
};
#endif
