// Handle information about conferences with unread texts.
//
// $Id: unread-confs.h,v 1.19 1995/12/30 16:26:16 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef KOMXX_UNREAD_CONFS_H
#define KOMXX_UNREAD_CONFS_H

#include <iostream.h>
#include <DLList.h>
#include "array.h"
#include "session.h"
#include "read-order.h"

class get_unread_confs_question;

struct Unread_info {
    Conf_no   conf;
    long      min_unread;	// At least this many unread texts.
    long      max_unread;	// At most this many unread texts.
				// -1 if no upper limit can be computed yet.
    Unread_info();
    Unread_info &operator =(const Unread_info &);
};


// Which conferences do we have unread texts in?

class Unread_confs {
  public:
    Unread_confs(Session *sess, Pers_no the_reader);
    Unread_confs(const Unread_confs &);
    ~Unread_confs();

    // This method can be used to require a certain amount of
    // knowledge about what is unread. Prefetching is done according
    // to the the criteria in the Work_quota argument.
    // Returns st_ok if so much already was prefetched that nothing
    // new needed to be done.
    Status prefetch(Work_quota *work);

    // goto_next_conf() will perform a pepsi(). It will return 0 if
    // there has been a fatal error (such as a lost connection).
    Conf_no              goto_next_conf();
    Status		 goto_conf(Conf_no);

    // Returns PROMPT_ERROR if there is a fatal error.
    Prompt               next_prompt();

    // Returns 0 if there is a fatal error, or if there are no more
    // texts to be read.
    Text_no              next_text(bool remove=false);
    void                 remove_text(Text_no);
    Conf_no              current_conf();
    Array<Unread_info>   confs_with_unread();
    long                 min_unread_texts_in_current();
    long                 max_unread_texts_in_current();

    Read_order_iterator current_conf_begin(int wanted_texts =100);
    Read_order_iterator current_conf_end();

    // Join a conference, or change priority/placement of an already
    // joined conference.  This will use Session::add_member to
    // actually add the member to the conference.
    Status               join_conference(Conf_no, unsigned char priority,
					 unsigned short placement);

    // The cache automatically registers this callback method with the
    // Async_dispatcher.  This method should not be used in any other
    // way.
    void        new_text(const Async_text_created &msg);
  private:
    const Unread_confs & operator = (const Unread_confs &); // NYI

    // Add a new conference. If LAST is true, append it to the list
    // of unread conferences, otherwise prepend it.
    void                 add_conference(Conf_no, bool last);

    // Utility functions that prefetch one known text/the list of
    // conferences that may have unread texts.
    Status need_one_mapped();
    Status need_unread_confs();

    Session              * session;
    Conferences          & conferences;
    Persons		 & persons;
    Maps		 & maps;
    Texts		 & texts;

    DLList<Read_order *>   unread_confs;
    // Since the Read_order objects contains state that is useful even
    // when we have read all texts, we save them here. +++ FIXME: The state
    // should be removed from them. The state that is useful is the
    // membership, and we really ought to have a cache for them.
    // The Read_orders on read_confs are by definition always empty.
    DLList<Read_order *>   read_confs;
    Pers_no		   reader;
    Conf_no		   current_working_conf;

    get_unread_confs_question *ucq;
    Async_handler_tag<Async_text_created> cr_tag;
    bool unread_confs_has_been_fetched;
};
#endif
