dnl $Id: mkmi.m4,v 1.9 1996/01/21 20:57:22 ceder Exp $
dnl Create Makefile.in from Makefile.src
dnl Copyright (C) 1994  Per Cederqvist and Inge Wallin
dnl
dnl This program is free software; you can redistribute it and/or modify
dnl it under the terms of the GNU General Public License as published by
dnl the Free Software Foundation; either version 2 of the License, or
dnl (at your option) any later version.
dnl 
dnl This program is distributed in the hope that it will be useful,
dnl but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl GNU General Public License for more details.
dnl 
dnl You should have received a copy of the GNU General Public License
dnl along with this program; if not, write to the Free Software
dnl Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
dnl
dnl Turn off output
divert(-1)
# Now, when the ouput is diverted to /dev/null, we can use the #
# comment-character with no ill effects.  This is, in my opinion,
# easier to read. Besides, blank lines can now be entered.

changequote([,])

# It is sometimes necessary to add a level of quotes.
define([LK_QUOTE],[[[$1]]])

# Append more text to a macro
define([LK_append],[define([$1],LK_QUOTE($1)[$2])])

# Loop over a (possibly empty) variable.  Use like this:
#   LK_LOOP([$(PROGS)],[$(INSTALL) $$i $(bindir)/$$i])
define([LK_LOOP],[[for i in .. $1; do \
	    if [ x$$i != x.. ]; then \
		if ( $2 ); then true; \
		else exit 1; fi; \
	    else true; fi; \
	done;]])

# A target ``foo'' often requires that certain commands be done in the
# current directory, and that ``foo'' should be preformed in all
# subdirectories as well.
#
# LK_descend(cmd, deps, localcmd, postlocalcmd) expands to
#
#	.PHONY: cmd
#	cmd: deps
#	localcmd
#		[code to perform cmd in lk_subdirs]
#	postlocalcmd
#
# localcmd should typically be empty or contain commands to perform
# cmd in the current directory.  All lines in localcmd should begin
# with a tab.  The code to perform cmd in lk_subdirs is only emitted
# if lk_subdirs is non-empty.  lk_subdirs is normally set up with
# LK_STD, and it should be a space-separated list.
#
# LK_descend can take an optional forth argument.  That is how
# ``make do-depend'' can ``make depend'' in subdirectories.

define([LK_descend], [[.PHONY: $1
$1: $2
$3]ifelse(lk_subdirs,,,LK_cmd([@for i in ]lk_subdirs[; do \
		if (cd ./$$i; $(MAKE) $1); \
		then true; else exit 1; fi ; \
	done]))dnl
ifelse($4,,,[$4]
)])

# LK_cmd(foo) expands to foo, but with a leading tab.  It
# is useful when outputting a make command line.

define([LK_cmd],[[	$1
]])


# Targets ``clean'' and friends.
#
# pure:  delete everything that can be recreated (an extension to GCS)
# realclean:  delete everything that can be recreated by make
# distclean:  delete everything that is not distributed
# clean:  delete build files (but not config files)
# mostlyclean:  delete most things

# ``do*'' expands to generic commands for performing the clean
# operation.  These are defined here, and not hard-coded in LK_CLEAN
# below, so that they can be overridden in Makefile.src.

define([dopure],[LK_cmd([$(RM) $(GENERIC-PURE)])])dnl
define([dorealclean],)dnl
define([dodistclean],[LK_cmd([$(RM) $(GENERIC-DISTCLEAN)])])dnl
define([doclean],[LK_cmd([$(RM) $(GENERIC-CLEAN)])])dnl
define([domostlyclean],[LK_cmd([$(RM) $(GENERIC-MOSTLYCLEAN)])])dnl

# LK_CLEAN will output everything that is needed to get good ``*clean''
# targets.  Some makefiles should override the ``do*'' macros before
# using LK_CLEAN.

define([LK_CLEAN],[dnl
[# Output from LK_CLEAN
GENERIC-PURE = Makefile.in
GENERIC-DISTCLEAN = *~ .\#* TAGS dependencies Makefile
GENERIC-CLEAN = dependencies stamp-*
GENERIC-MOSTLYCLEAN = *.o lib*.a core Makefile.in2 *.pyc *.incl]
LK_descend([pure],[dopure])
LK_descend([realclean],[dorealclean])
LK_descend([distclean],[dodistclean])
LK_descend([clean],[doclean])
LK_descend([mostlyclean],[domostlyclean])
[dopure: dorealclean]
dopure
[dorealclean: dodistclean]
dorealclean
[dodistclean: doclean]
dodistclean
[doclean: domostlyclean]
doclean
[domostlyclean:]
domostlyclean
[# End output from LK_CLEAN]]
)

# Make depend
#
# LK_ALL_C_DEPEND checks dependencies for all C files in the directory.
# LK_ALL_CXX_DEPEND does likewise for all C++ files.
# LK_ALL_DEPEND checks both C and C++ files.  It can only be called if
#    there are both C and C++ files.
# LK_DEPEND must be used after any of the above.  It should be used in
#    all directories.
#
# ``predepend'' can be set to a list of files which needs to be
# generated before the ``make depend'' can run.

define([dodepend],)
define([donedepend],)
define([predepend],)
define([LK_c_depend],
  [LK_cmd([$(CC) $(DEPENDFLAG) $(ALL_CFLAGS) $(srcdir)/*.c >dependencies])])
define([LK_cxx_depend],
  [LK_cmd([$(CXX) $(XXDEPENDFLAG) $(ALL_CXXFLAGS) \
		$(srcdir)/*.cc >dependencies])])
define([LK_cxx_local_depend],
  [LK_cmd([$(CXX) $(XXDEPENDFLAG) $(ALL_CXXFLAGS) \
		*.cc >dependencies])])
define([LK_all_depend],
  [LK_cmd([$(CC) $(DEPENDFLAG) $(ALL_CFLAGS) $(srcdir)/*.c >dependencies
	$(CXX) $(XXDEPENDFLAG) $(ALL_CXXFLAGS) \
		$(srcdir)/*.cc >>dependencies])])


# Used in libisc/src/Makefile.src.
define([LK_ALL_C_DEPEND],[define([dodepend],[LK_c_depend])])

# Used in various places.
define([LK_ALL_CXX_DEPEND],[define([dodepend],[LK_cxx_depend])])

# Used in genclasses
define([LK_ALL_CXX_LOCAL_DEPEND],[define([dodepend],[LK_cxx_local_depend])])

# Used in tcl-clients
define([LK_ALL_DEPEND],[define([dodepend],[LK_all_depend])])


define([LK_DEPEND],[[dependencies:
	touch dependencies]

include dependencies

LK_descend([depend],predepend,dodepend,donedepend)])



define([doincludes],)dnl
define([dolibraries],)dnl
define([prelibraries],)dnl

# How to generate a library:
#  LK_STD_LIB(libfoo.a,foo.o bar.o)

define([LK_STD_LIB],
[[LIBNAME = $1
LIBOBJS = $2

$(LIBNAME): $(LIBOBJS)
	$(AR) $(ARFLAGS) $]][[@ $]][[?
	$(RANLIB) $]][[@]
])


# LK_DESCEND should normally be the last macro found
# in Makefile.src
define([LK_DESCEND],
[LK_DEPEND
LK_CLEAN])


# LK_STD(subdirs)
#

define([LK_STD],[define([lk_subdirs],$1)dnl
[# Begin LK_STD
srcdir = @srcdir@
top_srcdir = @top_srcdir@
VPATH = @srcdir@
SHELL = /bin/sh

prefix = @prefix@
exec_prefix = @exec_prefix@
# Where to install nilkom, tkom and komiwish
bindir = @bindir@
# Where to install lib*.a.
libdir = @libdir@
# Location of TCL files for nilkom and tkom.
# If you want to run nilkom or tkom without installing them,
# you can specify --enable-run-in-place when you run the configure
# script.  That will alter the vale of datadir.
datadir=@datadir@
infodir=$(prefix)/info

CC = @CC@
CFLAGS = @CFLAGS@
ALL_CFLAGS = $(DEFS) $(CFLAGS) -I. -I@srcdir@ $(INCLUDES)
DEPENDFLAG = @DEPENDFLAG@

CXX = @CXX@
CXXFLAGS = @CXXFLAGS@
ALL_CXXFLAGS = $(DEFS) $(CXXFLAGS) -I. -I@srcdir@ $(INCLUDES)
XXDEPENDFLAG = @XXDEPENDFLAG@

INSTALL = @INSTALL@
INSTALL_DATA = @INSTALL_DATA@
INSTALL_PROGRAM = @INSTALL_PROGRAM@
MKDIR = mkdir
RANLIB = @RANLIB@
RM = rm -f
TOUCHLIB = @RANLIB@

VERSION = @KOMXX_VERSION@

DEFS = @DEFS@
LDFLAGS = @LDFLAGS@

.c.o:
	$(CC) -c $(CPPFLAGS) $(ALL_CFLAGS) $<

.cc.o:
	$(CXX) -c $(CPPXXFLAGS) $(ALL_CXXFLAGS) $<
# END section from LK_STD
]])
divert
