# $Id: komtext.py,v 1.4 1996/05/08 07:51:44 ceder Exp $
# Copyright (C) 1996  Per Cederqvist and Inge Wallin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import komhtml
import komxx
import pickle
from language import lingo

def prefetch_author(session, text, unprefetched_authors):
   if text.prefetch() == komxx.st_ok:
      session.conf(text.author()).prefetch_name()
   else:
      unprefetched_authors.append(text)

def safe_name(session, conf_no):
   try:
      return komhtml.conf_as_link(conf_no, session.conf(conf_no).name(),
				  session.my_login())
   except komxx.err_conf_zero:
      return None
   except (komxx.err_undef_conf, komxx.err_undef_pers,
	   komxx.err_access, komxx.err_perm):
      return lingo.undef_conf(conf_no)

def safe_name_direct_conf(session, conf_no):
   try:
      return komhtml.conf_as_direct_link(conf_no, 
					 session.conf(conf_no).name(),
					 session.my_login())
   except komxx.err_conf_zero:
      return None
   except (komxx.err_undef_conf, komxx.err_undef_pers,
	   komxx.err_access, komxx.err_perm):
      return lingo.undef_conf(conf_no)

def safe_author(session, text_no):
   try:
      return session.conf(session.text(text_no).author()).name()
   except (komxx.err_no_such_text):
      return lingo.nonexisting_text()
   except komxx.err_conf_zero:
      return lingo.anonymous_author()
   except (komxx.err_undef_conf, komxx.err_undef_pers):
      return lingo.nonexistent_author(session.text(text_no).author())

def prefetch_for_display(session, text):
   author=session.conf(text.author())
   author.prefetch_name()
   for rcpt in text.recipients():
      session.conf(rcpt.recipient()).prefetch_name()
      if rcpt.sender() != 0:
	 session.conf(rcpt.sender()).prefetch_name()
   unprefetched_authors=[]
   for uplink in text.uplinks():
      prefetch_author(session, session.text(uplink.parent()),
		      unprefetched_authors)
      if uplink.sender() != 0:
	 session.conf(uplink.sender()).prefetch_name()
   for footnote in text.footnotes_in():
      prefetch_author(session, session.text(footnote), unprefetched_authors)
   for comment in text.comments_in():
      prefetch_author(session, session.text(comment), unprefetched_authors)
   print "unprefetched_authors: " + `unprefetched_authors`
   for t in unprefetched_authors:
      if t.exists():
	 session.conf(t.author()).prefetch_name()

def map_for_display(session, text):
   map={}
   map["text_no"]       = text.text_no()
   map["creation_date"] = lingo.format_date(text.creation_time())
   map["lines"]         = text.num_lines()
   map["author"]        = safe_author(session, text.text_no())
   tmp=""
   for rcpt in text.recipients():
      tmp = tmp + lingo.format_recipient(
	 rcpt.is_carbon_copy(),
	 safe_name(session, rcpt.recipient()),
	 rcpt.local_no(),
	 rcpt.has_rec_time() and rcpt.rec_time(),
	 rcpt.has_sent_at() and rcpt.sent_at(),
	 safe_name(session, rcpt.sender()))
   map["recipients"] = tmp
   tmp = ""
   for uplink in text.uplinks():
      tmp = tmp + lingo.format_uplink(
	 uplink.is_footnote(),
	 komhtml.text_as_link(uplink.parent()),
	 safe_author(session, uplink.parent()),
	 uplink.sent_later() and uplink.sent_at(),
	 safe_name(session, uplink.sender()))
   map["uplinks"] = tmp
   map["num_marks"] = text.num_marks()
   map["subject"] = text.subject()
   map["body"] = komhtml.body_as_html(text.body())
   tmp=""
   for footnote in text.footnotes_in():
      tmp = tmp + lingo.format_footnote(komhtml.text_as_link(footnote),
					safe_author(session, footnote))
   map["footnotes"] = tmp
   tmp = ""
   for comment in text.comments_in():
      tmp = tmp + lingo.format_comment(komhtml.text_as_link(comment),
				       safe_author(session, comment))
   map["comments"] = tmp
   return map

def map_insert_recipients(session, map, recipients):
   sequence=0
   tmp=""
   for rcpt in recipients:
      tmp = tmp + lingo.write_text_format_recipient(
	 rcpt, safe_name(session, rcpt), sequence)
      sequence = sequence + 1
   map["recipients"] = tmp
   map["no_of_rcpts"] = pickle.dumps(sequence)

def target_conference(session, conf_no):
   conf = session.conf(conf_no)
   if conf.type().is_original():
      conf = session.conf(conf.super_conf())
   i=0
   # FIXME: enable code below when session.is_member is implemented.
   while i < 0 and not session.is_member(session.my_login(),
					 conf.permitted_submitters()):
      conf = session.conf(conf.super_conf())
      i=i+1
   return conf.conf_no()

def create_comment_map(session, tno):
   map={}
   text = session.text(tno)
   map["parent"]       = pickle.dumps(tno)
   map["parent_link"]  = komhtml.text_as_link(tno)
   map["parent_author"]= safe_author(session, tno)
   map["select_swascii"] = "checked selected"
   map["select_none"] = ""
   me_ismember=0
   author_ismember=0
   recipients = []
   for rcpt in text.recipients():
      if not rcpt.is_carbon_copy():
	 r = target_conference(session, rcpt.recipient())
	 if r not in recipients:
	    recipients.append(r)
	 if (me_ismember == 0 and
	     session.is_member(session.my_login(), rcpt.recipient())):
	    me_ismember = 1
	 if (author_ismember == 0 and
	     session.is_member(text.author(), rcpt.recipient())):
	    author_ismember = 1
   if not me_ismember:
      recipients.append(session.my_login())
   if not author_ismember:
      recipients.append(text.author())
   map_insert_recipients(session, map, recipients)

   map["subject"] = text.subject()
   map["body"] = ""
   comments = text.comments_in() + text.footnotes_in()
   map["known_comments"] = pickle.dumps(comments)
   return map

def create_text_map(session, cno_list):
   map={}
   map["conf_name"] = safe_name(session, cno_list[0])
   map["select_swascii"] = "checked selected"
   map["select_none"] = ""
   me_ismember=0
   recipients = []
   for rcpt in cno_list:
      r = target_conference(session, rcpt)
      if r not in recipients:
	 recipients.append(r)
	 if (me_ismember == 0 and session.is_member(session.my_login(), rcpt)):
	    me_ismember = 1
   if not me_ismember:
      recipients.append(session.my_login())
   map_insert_recipients(session, map, recipients)

   map["subject"] = ""
   map["body"] = ""
   return map

def recreate_comment_map(session, form_map, comments, recipients):
   # form_map is the data received from a text creation form.
   # This returns a new map, suitable to pass to language.write_comment.
   tno = pickle.loads(form_map["parent"])
   text = session.text(tno)
   map = {}
   map["parent"] = pickle.dumps(tno)
   map["parent_link"]  = komhtml.text_as_link(tno)
   map["parent_author"]= safe_author(session, tno)
   map["known_comments"] = pickle.dumps(comments)
   map_insert_recipients(session, map, recipients)
   map["subject"] = form_map["subject"]
   map["body"] = form_map["body"]
   map["select_swascii"] = ""
   map["select_none"] = ""
   if form_map["conversion"] == "swascii":
      map["select_swascii"] = "checked selected"
   else:
      map["select_none"] = "checked selected"
   return map
