# $Id: lm.py,v 1.8 1996/05/08 07:51:46 ceder Exp $
# Copyright (C) 1996  Per Cederqvist and Inge Wallin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# Login manager
# This is a login manager for WWW-kom, a special-purpose HTTP server.

import socket
import myhttpbase
import base64
import komxx
import string
import myserver
import komconfig
import sys
import komhtml
from language import lingo

# Global variables:
session=None
session_map={}
reverse_session_map={}
startport=None

# Constants used internally in this module:
true=1
false=0

internal_error='internal_error'

class lm(myhttpbase.myhttpbase):
   base = myhttpbase.myhttpbase

   def connect(self, host, port):
      """Connect to a remot port.

      Returns a tuple (status, readfile, writefile) where status
      is true if the connection succeeded, false if the connection
      was refused.  If the call succeeded, readfile will be a file
      opened for reading and writefile a file opened for writing."""

      s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
      try:
	 s.connect(host, port)
      except socket.error, x:
	 if x[1] == "Connection refused":
	    return (false, false, false)
	 else:
	    raise socket.error, x
      r = s.makefile("r");
      w = s.makefile("w");
      s.close()
      return (true, r, w)

   def start_server(self):
      global session_map
      global reverse_session_map
      # FIXME: configurable.  Try alternate hosts for load balancing.
      print "Starting a server..."
      sys.stdout.flush()
      host = "localhost"
      (success, r, w) = self.connect(host, startport)
      if success:
	 response = r.readline()
	 # FIXME: check errors
	 list = string.split(response)
	 if len(list) != 2 or list[0] != "WWW-kom":
	    self.log_error("Cannot contact to WWW-kom; got response \""
			   + response + "\"")
	    raise internal_error
	 port = string.atoi(list[1])
	 session_map[self.url.pers_no] = (host, port)
	 print "Created a server at", host, port
	 if reverse_session_map.has_key((host, port)):
	    print "This port was previously used by", reverse_session_map[(host, port)]
	    del session_map[reverse_session_map[(host, port)]]
	 sys.stdout.flush()
	 reverse_session_map[(host, port)] = self.url.pers_no	
	 r.close()
	 w.close()
	 self.connect_server()
      else:
	 self.log_error("Cannot contact WWW-kom; connection refused")
	 raise internal_error


   def connect_server(self):
      global session_map
      if session_map.has_key(self.url.pers_no):
	 (host, port) = session_map[self.url.pers_no]
	 (success, r, w) = self.connect(host, port)
	 if success:
	    print "Successfully connected to server at", host, port
	    sys.stdout.flush()
	    self.session_server_r = r
	    self.session_server_w = w
	 else:
	    print "Server at", host, port, "has died.  Removing records."
	    sys.stdout.flush()
	    del session_map[self.url.pers_no]
	 return success
      else:
	 return false

   def forward_query(self):
      print "Forwarding req", string.join([self.command, self.path, self.request_version])
      sys.stdout.flush()
      self.session_server_w.write(
	 string.join([self.command, self.path, self.request_version]))
      self.session_server_w.write("\r\n")
      print "Forwarding headers...", self.headers.headers
      sys.stdout.flush()
      self.session_server_w.writelines(self.headers.headers)
      print "Forwarding endline..."
      sys.stdout.flush()
      self.session_server_w.write("\r\n")
      # Forward the body, if any
      print "Forwarding flush..."
      sys.stdout.flush()
      self.session_server_w.flush()

   def server_side_redirect(self):
      if not self.connect_server():
	 self.start_server()
      self.forward_query()
      while 1:
	 line = self.session_server_r.readline()
	 print "Got a response:", line,
         sys.stdout.flush()
	 if line:
	    self.wfile.write(line)
	 else:
	    break
      self.session_server_r.close()
      self.session_server_w.close()

   def do_POST(self):
      length = self.headers.getheader('Content-length')
      if (self.headers.getheader('Content-type')
	  != "application/x-www-form-urlencoded"):
	 self.log_error("Bad content-type"
			+ self.headers.getheader('Content-type'))
	 self.send_error(400)
	 return
      body = self.rfile.read(eval(length))
      args = komhtml.form_data(body)
      u = string.split(self.path, '/')
      if len(u) != 2 or u[0] != '':
	 self.send_error(404)
	 return
      mname = "do_POST_" + u[1]
      method = getattr(self, mname)
      method(args)

   def do_POST_create(self, args):
      if not (args.has_key('username')
	      and args.has_key('password')
	      and args.has_key('passcheck')):
	 self.log_error("Missing argument to create")
	 self.error(400)
	 return
      if args['password'] != args['passcheck']:
	 self.respond(lingo.different_passwords())
      else:
	 name = args['username']
	 pno = session.create_person(name, args['password'])
	 if (pno == komxx.st_error):
	    self.respond(lingo.person_creation_failed(name))
	 else:
	    session.logout()
	    self.respond(lingo.creation_done(name, pno))
      
   def do_GET(self):
      try:
	 self.url = komhtml.decodeurl(self.path)
	 auth = self.headers.getheader('Authorization')
	 if (auth == None):
	    raise komhtml.bad_auth
	 if (self.url.pers_no):
	    self.server_side_redirect()
	 else:
	    [scheme, cookie] = string.split(auth)
	    if (string.lower(scheme) != "basic"):
	       self.log_error("Bad scheme")
	       self.send_error(500)
	       return
	    [self.user, self.password] \
			= string.split(base64.decodestring(cookie), ":")
	    matches = session.lookup_z_name(self.user, true, false)
	    if len(matches) == 0:
	       raise komhtml.bad_auth
	    elif len(matches) == 1:
	       # XXX FIXME: Hr grs en redirect innan vi vet att
	       # lsenordet r OK.  Dumt.  Om anvndaren skrev fel
	       # i namnet kommer han nu inte att kunna fixa det...
	       self.redirect(matches[0].conf_no())
	    else:
	       self.respond(lingo.ambiguous_login(self.user, matches))
      except komhtml.bad_auth:
	 self.bad_auth()
      except komhtml.not_a_komurl, arg:
	 if self.path[-1] == "/":
	    self.path = self.path + "index.html"
	 self.base.do_GET(self)
      except internal_error:
	 self.send_error(500)



def test():
   global session
   global startport

   startport = string.atoi(sys.argv[3])
   print "Connecting to LysKOM host", sys.argv[1] + ", port",
   print sys.argv[2] + ". Session started via port", startport
   sys.stdout.flush()
   try:
      session = komxx.new_session(sys.argv[1], sys.argv[2])
   except komxx.no_connection:
      sys.stderr.write("Unable to connect to " + sys.argv[1] + ":"
		       + sys.argv[2] + "\n")
      sys.stderr.flush()
      sys.exit(1)
   print "Connected to LysKOM server.  Starting HTTP server"
   sys.stdout.flush()
   try:
      httpd = myserver.myserver(('', string.atoi(komconfig.wwwport)), lm)
      print "HTTP server started."
      sys.stdout.flush()
      httpd.serve_forever()
   except komxx.no_connection:
      sys.stderr.write("Connection to " + sys.argv[1] + ":" + sys.argv[2] 
		       + "lost\n")
      sys.exit(2)

if __name__ == '__main__':
   # usage: lm kom-server-host kom-port-number wwwkom-port-number
   test()
