# $Id: toolbar.py,v 1.3 1996/05/08 07:51:50 ceder Exp $
# Copyright (C) 1996  Per Cederqvist and Inge Wallin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import komhtml
import string
from language import lingo

system_error='system_error'

class toolbar:
   """Toolbar: the line of icons at the top (and bottom) of each page.

   The toolbar has this look, from left to right:

   default: typically "read next text"
   std:    go home, go conf, etc
   mode-special: e.g. write comment
   kill:    restart with force"""

   def __init__(self, session):
      self.session = session
      self.top_wanted = 1
      self.bottom_wanted = 0
      self.default = []
      self.std = []
      self.toolbar = []

   def fix_std(self):
      if not self.std:
	 self.std = [komhtml.icon("home.gif", lingo.alt_home(),
				  komhtml.komurl(self.session, "home")),
		     komhtml.icon("selconf.gif", lingo.alt_select_conf(),
				  komhtml.komurl(self.session,
						 "select_conf_form"))]
	 self.kill = [komhtml.icon("exit.gif", lingo.alt_kill(),
				   komhtml.komurl(self.session, "kill"))]

   def enable_bottom(self):
      self.bottom_wanted = 1

   def disable_top(self):
      self.top_wanted = 0

   def add_tool(self, image, alt, url):
      self.toolbar.append(komhtml.icon(image, alt, url))

   def add_dflt(self, image, alt, url):
      if len(self.default) != 0:
	 raise system_error, "default set twice"
      self.default = [komhtml.icon(image, alt, url)]

   def add_kill(self):
      self.add_tool("kill.gif", "Avsluta",
		    komhtml.komurl(self.session, "kill"))

   def add_comment(self, textno):
      self.add_tool("comment.gif", lingo.alt_comment(),
		    komhtml.komurl(self.session, 
				   "comment?parent=" + `textno`))

   def add_current_conf(self, confno):
      self.add_tool("board.gif", lingo.alt_conf(),
		    komhtml.komurl(self.session, 
				   "conf?conf_no=" + `confno`))

   def add_write(self, confno):
      self.add_tool("write.gif", lingo.alt_write_text(),
		    komhtml.komurl(self.session, 
				   "write_text?conf_no=" + `confno`))
		    

   def add_default_conf(self, cno):
      self.add_dflt("next.gif", lingo.alt_next_conf(),
		    komhtml.komurl(self.session, "conf?conf_no=" + `cno`))

   def add_default_text(self, tno):
      self.add_dflt("next.gif", lingo.alt_next_text(),
		    komhtml.komurl(self.session, "text?text_no=" + `tno`))

   def add_default_time(self):
      self.add_dflt("next.gif", lingo.alt_see_time(),
		    komhtml.komurl(self.session, "time"))

   def at_top(self):
      if self.top_wanted:
	 return self.as_string()
      else:
	 return ""

   def as_string(self):
      self.fix_std()
      return "<p>" + string.join(self.default + self.std
				 + self.toolbar + self.kill, "\n") + "\n<p>"
      

   def at_bottom(self):
      if self.bottom_wanted:
	 return self.as_string()
      else:
	 return ""
